﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media.Animation;

namespace PRScreenWithKinect.UI.Utility
{
    public class ScrollWithGesture : ScrollViewer
    {
        private const int TIMESLIDE = 500;
        private const double ACCERATION = 0.5;
        //Register a DependencyProperty which has a onChange callback
        public static DependencyProperty CurrentVerticalOffsetProperty = DependencyProperty.Register("CurrentVerticalOffset", typeof(double), typeof(ScrollWithGesture), new PropertyMetadata(new PropertyChangedCallback(OnVerticalChanged)));
        public static DependencyProperty CurrentHorizontalOffsetProperty = DependencyProperty.Register("CurrentHorizontalOffsetOffset", typeof(double), typeof(ScrollWithGesture), new PropertyMetadata(new PropertyChangedCallback(OnHorizontalChanged)));

        //When the DependencyProperty is changed change the vertical offset, thus 'animating' the scrollViewer
        private static void OnVerticalChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ScrollWithGesture viewer = d as ScrollWithGesture;
            viewer.ScrollToVerticalOffset((double)e.NewValue);
        }

        //When the DependencyProperty is changed change the vertical offset, thus 'animating' the scrollViewer
        private static void OnHorizontalChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ScrollWithGesture viewer = d as ScrollWithGesture;
        
                viewer.ScrollToHorizontalOffset((double)e.NewValue);
            
            
        }


        public double CurrentHorizontalOffset
        {
            get { return (double)this.GetValue(CurrentHorizontalOffsetProperty); }
            set { this.SetValue(CurrentHorizontalOffsetProperty, value); }
        }

        public double CurrentVerticalOffset
        {
            get { return (double)this.GetValue(CurrentVerticalOffsetProperty); }
            set { this.SetValue(CurrentVerticalOffsetProperty, value); }
        }
        public void ScrollToPosition(double x, double y)
        {
            DoubleAnimation vertAnim = new DoubleAnimation();
            vertAnim.From = this.VerticalOffset;
            vertAnim.To = y;
            vertAnim.DecelerationRatio = ACCERATION;
            vertAnim.Duration = new Duration(TimeSpan.FromMilliseconds(TIMESLIDE));
            DoubleAnimation horzAnim = new DoubleAnimation();
            horzAnim.From = this.HorizontalOffset;
            horzAnim.To = x;
            horzAnim.DecelerationRatio = ACCERATION;
            horzAnim.Duration = new Duration(TimeSpan.FromMilliseconds(TIMESLIDE));
            Storyboard sb = new Storyboard();
            sb.Children.Add(vertAnim);
            sb.Children.Add(horzAnim);
            Storyboard.SetTarget(vertAnim, this);
            Storyboard.SetTargetProperty(vertAnim, new PropertyPath(ScrollWithGesture.CurrentVerticalOffsetProperty));
            Storyboard.SetTarget(horzAnim, this);
            Storyboard.SetTargetProperty(horzAnim, new PropertyPath(ScrollWithGesture.CurrentHorizontalOffsetProperty));
            
            sb.Begin();
        }
        public void ScrollToPositionWithoutAnimate(double x, double y)
        {
            DoubleAnimation vertAnim = new DoubleAnimation();
            vertAnim.From = this.VerticalOffset;
            vertAnim.To = y;
            vertAnim.DecelerationRatio = ACCERATION;
            vertAnim.Duration = new Duration(TimeSpan.FromMilliseconds(0));
            DoubleAnimation horzAnim = new DoubleAnimation();
            horzAnim.From = this.HorizontalOffset;
            horzAnim.To = x;
            horzAnim.DecelerationRatio = ACCERATION;
            horzAnim.Duration = new Duration(TimeSpan.FromMilliseconds(0));
            Storyboard sb = new Storyboard();
            sb.Children.Add(vertAnim);
            sb.Children.Add(horzAnim);
            Storyboard.SetTarget(vertAnim, this);
            Storyboard.SetTargetProperty(vertAnim, new PropertyPath(ScrollWithGesture.CurrentVerticalOffsetProperty));
            Storyboard.SetTarget(horzAnim, this);
            Storyboard.SetTargetProperty(horzAnim, new PropertyPath(ScrollWithGesture.CurrentHorizontalOffsetProperty));
            sb.Begin();
        }

    }
}
